from __future__ import annotations

from pathlib import Path

from flask import Flask, current_app
from sqlalchemy.testing.suite.test_reflection import users
from sqlmodel import SQLModel, Session

from hogwarts_testing_platform.model.report import Report
from hogwarts_testing_platform.model.testcase import TestCase
from hogwarts_testing_platform.model.user import User
from hogwarts_testing_platform.router.case_router import CaseRouter
from hogwarts_testing_platform.service.db_router import DBItemAPI
from hogwarts_testing_platform.service.demo import DemoView
from hogwarts_testing_platform.service.login import LoginView
from hogwarts_testing_platform.service.ui_item_router import UIItemAPI
from hogwarts_testing_platform.site.db import engine


class Server:
    def __init__(self):
        self.app: Flask | None = None

    def get_app(self) -> Flask:
        if self.app is None:
            app = Flask(
                __name__,
                # 模板目录
                template_folder=Path(__file__).parent.parent / 'templates'
            )

            @app.before_request
            def auth():
                current_app.logger.info('app auth')

            self.app = app
            # self.app = OpenAPI(__name__)
            self.app.debug = True

            # json输出支持中文
            self.app.config['JSON_AS_ASCII'] = False

            # 用于session加密
            self.app.config['SECRET_KEY'] = 'hogwarts'

            # 注册路由
            self.include_routes()

        return self.app

    def init_db(self):
        # 清理数据
        SQLModel.metadata.drop_all(engine)
        SQLModel.metadata.create_all(engine)

        with Session(engine) as session:
            admin = User()
            admin.username = 'admin'
            print(admin.model_dump())
            session.add(admin)

            case1 = TestCase()
            case1.name = 'demo'
            case1.user = admin

            session.add(case1)

            session.commit()

    def include_routes(self):
        """
        所有路由的注册
        :return:
        """

        # for path, model in {
        #     '/bug': Bug,
        #     '/testcase': TestCase,
        #     '/report': Report,
        # }.items():
        #     name = path[1:]
        #     bug_view_func = ItemAPI.as_view(name, model)
        #     self.get_app().add_url_rule(rule=f'{path}/<int:id>', view_func=bug_view_func, methods=['GET'])
        #     self.get_app().add_url_rule(rule=f'{path}/', view_func=bug_view_func, methods=['GET'])
        #     self.get_app().add_url_rule(rule=f'{path}', view_func=bug_view_func, methods=['POST'])
        #

        # db示例
        for path, model in {
            '/db/user': User,
            # '/db/testcase': TestCase,
            # '/db/report': Report,
        }.items():
            name = path[1:]
            view_func = DBItemAPI.as_view(name, model)
            self.get_app().add_url_rule(rule=f'{path}/<int:id>', view_func=view_func, methods=['GET', 'DELETE'])
            self.get_app().add_url_rule(rule=f'{path}/', view_func=view_func, methods=['GET'])
            self.get_app().add_url_rule(rule=f'{path}', view_func=view_func, methods=['POST'])

        # 模板示例
        for path, model in {
            '/user': User,
            '/testcase': TestCase,
            '/report': Report,
        }.items():
            # 取出来 user testcase report bug
            name = ''.join(path.rpartition('/'))
            # 把MethodView初始化出来，并封装到一个函数里
            view_func = UIItemAPI.as_view(name, model)
            # 注册路由
            self.get_app().logger.debug(f"{path} {view_func}")
            # 单个数据的删除修改查询
            self.get_app().add_url_rule(
                rule=f'{path}/<int:id>.<suffix>',
                view_func=view_func,
                methods=['GET', 'DELETE']
            )
            # 列举所有数据
            self.get_app().add_url_rule(rule=f'{path}/', view_func=view_func, methods=['GET'])
            # 数据的新增
            self.get_app().add_url_rule(rule=f'{path}', view_func=view_func, methods=['POST'])

        # 方便调试，注册一个demo接口
        self.get_app().add_url_rule(rule='/demo/<path>', view_func=DemoView.as_view('demo'))
        self.get_app().add_url_rule(rule='/login', view_func=LoginView.as_view('login'))

        case_view_func = CaseRouter.as_view(name='case')
        self.get_app().add_url_rule(rule='/v1/testcase/<int:id>', view_func=case_view_func)
        self.get_app().add_url_rule(rule='/v1/testcase', view_func=case_view_func, methods=['GET', 'POST'])

    def run(self, port=None):
        self.get_app().run(port=port, debug=True)
